"""Declares :class:`RealmConfigSchema`."""
import marshmallow
from unimatrix.lib.datastructures import DTO


class RealmSingleSignonSchema(marshmallow.Schema):
    dict_class = DTO

    providers = marshmallow.fields.Field(
        missing=dict,
        default=dict
    )


class RealmConfigSchema(marshmallow.Schema):
    dict_class = DTO

    public = marshmallow.fields.Boolean(
        missing=False,
        default=False
    )

    domains = marshmallow.fields.List(
        marshmallow.fields.String,
        missing=list,
        default=list
    )

    sso = marshmallow.fields.Nested(
        RealmSingleSignonSchema,
        missing=lambda: RealmSingleSignonSchema().load({}),
        default=lambda: RealmSingleSignonSchema().load({})
    )



#
# Copyright (C) 2019-2020 Cochise Ruhulessin
#
# This file is part of python-iam.
#
# python-iam is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# python-iam is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-iam.  If not, see <https://www.gnu.org/licenses/>.
"""Declares :class:`~PhonenumberPrincipal`."""
from .principal import Principal


class PhonenumberPrincipal(Principal):
    """A :term:`Principal` that identifies a :term:`Subject` using an
    value address.
    """
    resource_class = 'PhonenumberPrincipal'
    pattern = '^[+]*[(]{0,1}[0-9]{1,4}[)]{0,1}[-\s\./0-9]*$'

    def __init__(self, value):
        super().__init__()
        self.value = value

    def __str__(self):
        return self.value or ''



"""Declares :class:`SuccessURLMixin`."""
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.shortcuts import resolve_url
from django.utils.http import url_has_allowed_host_and_scheme


class SuccessURLMixin:
    redirect_field_name = REDIRECT_FIELD_NAME
    success_url_allowed_hosts = set()

    def get_success_url_allowed_hosts(self):
        return {self.request.get_host(), *self.success_url_allowed_hosts}

    def get_default_next_page(self):
        return None

    def get_next_page(self):
        default_next_page = self.get_default_next_page()
        if self.next_page is not None:
            next_page = resolve_url(self.next_page)
        elif default_next_page:
            next_page = resolve_url(default_next_page)
        else:
            next_page = self.next_page

        if (self.redirect_field_name in self.request.POST or
                self.redirect_field_name in self.request.GET):
            next_page = self.request.POST.get(
                self.redirect_field_name,
                self.request.GET.get(self.redirect_field_name)
            )
            url_is_safe = url_has_allowed_host_and_scheme(
                url=next_page,
                allowed_hosts=self.get_success_url_allowed_hosts(),
                require_https=self.request.is_secure(),
            )
            # Security check -- Ensure the user-originating redirection URL is
            # safe.
            if not url_is_safe:
                next_page = self.request.path
        return next_page



"""Declares context processors for use with Django."""
from django.conf import settings
from django.utils.functional import SimpleLazyObject

from iam.ext.django.models import PlatformAgreementLanguage


def policies(request):
    """Updates the context with the effective :term:`Terms of Service` and
    :term:`Privacy Policy`.
    """
    f = lambda: PlatformAgreementLanguage.objects.get_effective(
        settings.IAM_TERMS_OF_SERVICE)
    p = lambda: PlatformAgreementLanguage.objects.get_effective(
        settings.IAM_PRIVACY_POLICY)
    return {
        'iam_terms_of_service': SimpleLazyObject(f),
        'iam_privacy_policy': SimpleLazyObject(p)
    }


def context(request):
    """Adds common IAM context variables."""
    ctx = {
        'subject': request.user
    }
    ctx['iam_urls'] = {
        'forms': {
            'names': f'{settings.IAM_URL_NAMESPACE}:forms:names',
            'terms': f'{settings.IAM_URL_NAMESPACE}:forms:terms',
        },
        'login': f'{settings.IAM_URL_NAMESPACE}:login',
        'login_token': f'{settings.IAM_URL_NAMESPACE}:login.token',
        'logout': f'{settings.IAM_URL_NAMESPACE}:logout',
        'register': f'{settings.IAM_URL_NAMESPACE}:register',
        'resetcredentials': f'{settings.IAM_URL_NAMESPACE}:resetcredentials',
    }
    return ctx



"""Declares :class:`LogoutView`."""
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth import logout
from django.http import HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView
from django.views.decorators.cache import never_cache

from .successurlmixin import SuccessURLMixin


class LogoutView(SuccessURLMixin, TemplateView):
    next_page = None
    redirect_field_name = REDIRECT_FIELD_NAME
    template_name = 'registration/logged_out.html'
    extra_context = None

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        logout(request)
        next_page = self.get_next_page()
        if next_page:
            # Redirect to this page until the session has been cleared.
            return HttpResponseRedirect(next_page)
        return super().dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        """Logout may be done via POST."""
        return self.get(request, *args, **kwargs)

    def get_default_next_page(self):
        """Return the default next page if one isn't specified through the
        query parameter matching :attr:`redirect_field_name`.
        """
        return settings.LOGOUT_REDIRECT_URL



#!/usr/bin/env python3
#
# Copyright (C) 2019-2020 Cochise Ruhulessin
#
# This file is part of python-iam.
#
# python-iam is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# python-iam is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-iam.  If not, see <https://www.gnu.org/licenses/>.
from setuptools import find_packages
from setuptools import setup

requirements = str.split(open('requirements.txt').read(), '\n')

setup(
    name='python-iam',
    version='0.2.8',
    description='Identity and Access Management for Python',
    author='Cochise Ruhulessin',
    author_email='cochise.ruhulessin@digitalcitizen.nl',
    url='https://gitlab.com/unimatrixone/libraries/python-iam',
    install_requires=requirements,
    packages=find_packages(),
    include_package_data=True,
    license="GPLv3",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Django",
        "Framework :: Flask",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Clustering",
        "Topic :: System :: Distributed Computing"
    ]
)
